@@ -54,8 +54,8 @@ class Function(object):
         :param domain: list of points where the function is defined, equidistantly spaced!
         :param function_callable: callable function to evaluate this Function.
         """
-        if not self._is_evenly_spaced_domain(domain):
-            raise RuntimeWarning("Given domain is not equidistantly spaced")
+        # if not self._is_evenly_spaced_domain(domain):
+        #    raise RuntimeWarning("Given domain is not equidistantly spaced")
 
         if not isinstance(domain, numpy.ndarray):
             self._dom = numpy.array(domain)
@@ -86,6 +86,9 @@ class Function(object):
 
         return False
 
+    def is_complex(self):
+        return numpy.any(numpy.iscomplex(self.eval()))
+
     def is_evenly_spaced(self):
         return True
 
@@ -96,6 +99,35 @@ class Function(object):
         """
         return Function(self._dom, self._f)
 
+    def transform(self, transformation):
+        """
+        Transforms the function f based on the given transformation and returns a new Function F via:
+
+            F.domain = f.domain
+            F(x) = transformation(x, f(x)) for x in f.domain
+
+        The transformation has to accept two parameters: x and f(x)
+
+        :Example:
+        >>> # take the square of a function
+        >>> transformation = lambda x, fx: fx**2
+        >>> # ignoring the previous function and just return a straight line with slope 1.
+        >>> transformation = lambda x, fx: x
+        >>> # Scaling by x**2
+        >>> transformation = lambda x, fx: x**2 * fx
+
+        :param transformation: callable
+        :return: Function
+        """
+
+        if not callable(transformation):
+            raise RuntimeError("Transformation has to be callable")
+
+        if not transformation.__code__.co_argcount == 2:
+            raise RuntimeError("Transformation has to accept two parameters: x and f(x)")
+
+        return Function.to_function(self._dom, [transformation(x, fx) for (x, fx) in zip(self._dom, self.eval())])
+
     def reinterpolate(self):
         """
         Uses the internal callable function, to interpolate it on the given domain.
@@ -126,7 +158,7 @@ class Function(object):
     def scale_domain(self, factor):
         dom = factor * self._dom
         f = self._f
-        return Function(dom, lambda x: f(x/factor))
+        return Function(dom, lambda x: f(x / factor))
 
     def apply(self, function):
         """
@@ -283,46 +315,68 @@ class Function(object):
     def from_function(cls, fun: 'Function'):
         return cls.to_function(fun.get_domain(), fun.get_function())
 
-    def add(self, other):
-        return self.__add__(other)
-
     @staticmethod
     def _is_number(other):
         return (isinstance(other, int) or
                 isinstance(other, float) or
-                isinstance(other, numpy.complex)
-                or isinstance(other, numpy.float))
+                isinstance(other, numpy.complex) or
+                isinstance(other, numpy.float) or
+                (isinstance(other, numpy.ndarray) and other.size == 1))
+
+    @staticmethod
+    def _unknown_type(other):
+        raise RuntimeError("Unknown type of other")
 
     def __add__(self, other):
         if isinstance(other, Function):
             return Function(self._dom, lambda x: self._f(x) + other.get_function()(x))
+        if callable(other):
+            return Function(self._dom, lambda x: self._f(x) + other(x))
         if self._is_number(other):
             return Function(self._dom, lambda x: self._f(x) + other)
 
+        self._unknown_type(other)
+
     def __sub__(self, other):
         if isinstance(other, Function):
             return Function(self._dom, lambda x: self._f(x) - other.get_function()(x))
+        if callable(other):
+            return Function(self._dom, lambda x: self._f(x) - other(x))
         if self._is_number(other):
             return Function(self._dom, lambda x: self._f(x) - other)
 
+        self._unknown_type(other)
+
     def __pow__(self, power):
         if isinstance(power, Function):
             return Function(self._dom, lambda x: self._f(x) ** power.get_function()(x))
+        if callable(power):
+            return Function(self._dom, lambda x: self._f(x) ** power(x))
         if self._is_number(power):
             return Function(self._dom, lambda x: self._f(x) ** power)
 
+        self._unknown_type(power)
+
     def __mul__(self, other):
         if isinstance(other, Function):
             return Function(self._dom, lambda x: self._f(x) * other.get_function()(x))
+        if callable(other):
+            return Function(self._dom, lambda x: self._f(x) * other(x))
         if self._is_number(other):
             return Function(self._dom, lambda x: self._f(x) * other)
 
+        self._unknown_type(other)
+
     def __truediv__(self, other):
         if isinstance(other, Function):
             return Function(self._dom, lambda x: self._f(x) / other.get_function()(x))
+        if callable(other):
+            return Function(self._dom, lambda x: self._f(x) / other(x))
         if self._is_number(other):
             return Function(self._dom, lambda x: self._f(x) / other)
 
+        self._unknown_type(other)
+
     def __neg__(self):
         f = self._f
         return Function(self._dom, lambda x: -f(x))
@@ -359,6 +413,9 @@ class Function(object):
         if show:
             pylab.show()
 
+    def show(self):
+        self.plot(show=True)
+
     @property
     def real(self):
         return Function(self._dom, lambda x: self._f(x).real)
@@ -379,6 +436,12 @@ class Function(object):
         return roots
 
 
+class NullFunction(Function):
+    def __init__(self, domain):
+        super(NullFunction, self).__init__(domain, lambda x: 0)
+
+
+
 class UnevenlySpacedFunction(Function):
     @classmethod
     def _is_evenly_spaced_domain(cls, domain):
@@ -459,7 +522,6 @@ class Antiderivative(Integral):
 class Derivative(Function):
     @classmethod
     def to_function(cls, domain, feval):
-        # TODO: test
         feval = evaluate(domain, feval)
         fprime = numpy.gradient(feval, cls.get_dx(domain), edge_order=2)
         return Function.to_function(domain, fprime)
@@ -506,6 +568,15 @@ def set_interpolation_type(interpolation_type):
 
 
 def to_function(x_space, feval, interpolation=None, to_zero=True):
+    """
+    Returns an interpolated function using x and f(x).
+
+    :param x_space: domain of the function
+    :param feval: evaluated function at f(x) for each x/ or callable function
+    :param interpolation: Type of interpolation, see scipy.interp1d
+    :param to_zero: the returned function will evaluate to zero (or nan) outside the domain
+    :return: Callable function
+    """
     if interpolation is None:
         global FUNCTION_INTERPOLATION_TYPE
         interpolation = FUNCTION_INTERPOLATION_TYPE
@@ -525,7 +596,69 @@ def to_function(x_space, feval, interpolation=None, to_zero=True):
 
     real = scipy.interpolate.interp1d(x_space, feval.real, fill_value=fill, bounds_error=False,
                                       kind=interpolation)
-    imag = scipy.interpolate.interp1d(x_space, feval.imag, fill_value=fill, bounds_error=False,
-                                      kind=interpolation)
 
-    return lambda x: real(x) + 1j * imag(x)
+    if numpy.any(numpy.iscomplex(feval)):
+        imag = scipy.interpolate.interp1d(x_space, feval.imag, fill_value=fill, bounds_error=False,
+                                          kind=interpolation)
+
+        return lambda x: real(x) + 1j * imag(x)
+
+    return real
+
+
+class FunctionFileLoader:
+    """
+    Simple class to write a function to disk and read a function from disk.
+
+    Uses the numpy.savetxt/loadtxt methods.
+    """
+    def __init__(self, file):
+        self._file = file
+
+
+    def from_file(self):
+        """
+        Loads a function from file and returns its object.
+
+        This can read files of the row-form:
+            - x f(x).real f(x).imag
+            - x f(x).real
+
+        :return: Function
+        """
+
+        data = numpy.loadtxt(self._file)
+        try:
+            x, freal, fimag = data.T
+        except:
+            try:
+                x, freal = data.T
+                fimag = None
+            except:
+                raise RuntimeError("Unknown function file type")
+
+        if fimag is None:
+            return Function.to_function(x, freal)
+        else:
+            feval = freal + 1j * fimag
+            return Function.to_function(x, feval)
+
+    def to_file(self, function: Function, header=None):
+        """
+        Saves a given function to disk.
+
+        It will save the file in such a way that it is readable by from_file.
+
+        :param function: Function to save to disk
+        :param header: Header string attached at the beginning of the file, will be added as a comment
+        :return: None
+        """
+        domain = function.get_domain()
+        feval = function.eval()
+
+        if function.is_complex():
+            data = numpy.array([domain, feval.real, feval.imag])
+        else:
+            data = numpy.array([domain, feval])
+
+        numpy.savetxt(self._file, data.T, header=header)
